"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _isObject = _interopRequireDefault(require("../utils/object/isObject"));

var _warn = _interopRequireDefault(require("../utils/dev/warn"));

var _arrayFrom = _interopRequireDefault(require("../utils/array/arrayFrom"));

var _resolveOptions = _interopRequireDefault(require("../action-creators/helpers/resolveOptions"));

var _progressReducer = _interopRequireDefault(require("./helpers/progressReducer"));

var _configuration = require("../configuration");

var _fetchList = _interopRequireDefault(require("../actions/RESTful/fetchList"));

var _fetchItem = _interopRequireDefault(require("../actions/RESTful/fetchItem"));

var _newItem = _interopRequireDefault(require("../actions/RESTful/newItem"));

var _clearNewItem = _interopRequireDefault(require("../actions/RESTful/clearNewItem"));

var _editNewItem = _interopRequireDefault(require("../actions/RESTful/editNewItem"));

var _createItem = _interopRequireDefault(require("../actions/RESTful/createItem"));

var _editItem = _interopRequireDefault(require("../actions/RESTful/editItem"));

var _editNewOrExistingItem = _interopRequireDefault(require("../actions/RESTful/editNewOrExistingItem"));

var _clearItemEdit = _interopRequireDefault(require("../actions/RESTful/clearItemEdit"));

var _updateItem = _interopRequireDefault(require("../actions/RESTful/updateItem"));

var _destroyItem = _interopRequireDefault(require("../actions/RESTful/destroyItem"));

var _selectItem = _interopRequireDefault(require("../actions/selection/selectItem"));

var _selectAnotherItem = _interopRequireDefault(require("../actions/selection/selectAnotherItem"));

var _deselectItem = _interopRequireDefault(require("../actions/selection/deselectItem"));

var _clearSelectedItems = _interopRequireDefault(require("../actions/selection/clearSelectedItems"));

var _clearResource = _interopRequireDefault(require("../actions/clear/clearResource"));

var _clearItem = _interopRequireDefault(require("../actions/clear/clearItem"));

var _clearList = _interopRequireDefault(require("../actions/clear/clearList"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var STANDARD_REDUCERS = {
  fetchList: _fetchList["default"].reducer,
  fetchItem: _fetchItem["default"].reducer,
  newItem: _newItem["default"].reducer,
  clearNewItem: _clearNewItem["default"].reducer,
  editNewItem: _editNewItem["default"].reducer,
  createItem: _createItem["default"].reducer,
  editItem: _editItem["default"].reducer,
  editNewOrExistingItem: _editNewOrExistingItem["default"].reducer,
  clearItemEdit: _clearItemEdit["default"].reducer,
  updateItem: _updateItem["default"].reducer,
  destroyItem: _destroyItem["default"].reducer,
  clearResource: _clearResource["default"].reducer,
  clearItem: _clearItem["default"].reducer,
  clearList: _clearList["default"].reducer,
  selectItem: _selectItem["default"].reducer,
  selectAnotherItem: _selectAnotherItem["default"].reducer,
  deselectItem: _deselectItem["default"].reducer,
  clearSelectedItems: _clearSelectedItems["default"].reducer
};
var PROGRESS_COMPATIBLE_ACTIONS = {
  fetchList: true,
  fetchItem: true,
  updateItem: true,
  createItem: true
};

function getProgressReducer(key) {
  if (key === 'fetchList') {
    return function (resources, action) {
      return (0, _progressReducer["default"])(resources, action, 'lists');
    };
  } else {
    return _progressReducer["default"];
  }
}

function buildReducersDictionary(resourceOptions, actionsDictionary, actionsOptions) {
  var globalConfiguration = (0, _configuration.getConfiguration)();
  var reducersDict = Object.keys(actionsOptions).reduce(function (memo, key) {
    var actionOptions = actionsOptions[key];
    var reducer = (0, _isObject["default"])(actionOptions) && actionOptions.reducer || STANDARD_REDUCERS[key];

    if (reducer) {
      var reducerOptions = (0, _resolveOptions["default"])(globalConfiguration, resourceOptions, actionOptions, ['progress', 'beforeReducers', 'afterReducers']);

      if (reducerOptions.progress && (!STANDARD_REDUCERS[key] || PROGRESS_COMPATIBLE_ACTIONS[key])) {
        reducerOptions.beforeReducers = [].concat(_toConsumableArray(reducerOptions.beforeReducers), [getProgressReducer(key)]);
      }

      var _reducer = function () {
        if (reducerOptions.beforeReducers.length > 0 || reducerOptions.afterReducers.length > 0) {
          return function (resources, action) {
            var _resources = resources;
            reducerOptions.beforeReducers.forEach(function (beforeReducer) {
              _resources = beforeReducer(_resources, action);
            });
            _resources = reducer(_resources, action);
            reducerOptions.afterReducers.forEach(function (afterReducer) {
              _resources = afterReducer(_resources, action);
            });
            return _resources;
          };
        } else {
          return reducer;
        }
      }();

      memo[actionsDictionary[key]] = {
        options: reducerOptions,
        reducer: _reducer
      };
    } else {
      if (resourceOptions.localOnly && STANDARD_REDUCERS[key]) {
        (0, _warn["default"])("Action '".concat(key, "' is not compatible with the localOnly option."));
      } else {
        var standardReducersList = Object.keys(STANDARD_REDUCERS).join(', ');
        (0, _warn["default"])("Action '".concat(key, "' must match the list of standard reducers (").concat(standardReducersList, ") or define a 'reducer' option."));
      }
    }

    return memo;
  }, {});
  (0, _arrayFrom["default"])(resourceOptions.clearOn).forEach(function (action) {
    reducersDict[action] = {
      reducer: STANDARD_REDUCERS.clearResource
    };
  });
  (0, _arrayFrom["default"])(resourceOptions.reducesOn).forEach(function (_ref) {
    var action = _ref.action,
        reducer = _ref.reducer;
    reducersDict[action] = {
      reducer: reducer
    };
  });
  return reducersDict;
}

var _default = buildReducersDictionary;
exports["default"] = _default;